// Parallel version of correlation-function, or the "pair counting" algorithm
// Code is very similar to the sequential version
// Requires OpenMP to build/run:
// > g++ mul2.cpp -fopenmp

#include <iostream>
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <omp.h>

using namespace std;

int ptNum;		//total
int sampleNum;	
int intervalNum;	//num of intervals

int numSamples = 30; 	//A large enough constant that represents Central Limit Theory (change to 40?)
//!! change back
double error;
double **tempResult;
double *finalResult;

double real_time = 0;
double shuffle_time = 0;

float *PT_x;
float *PT_y;
float *PT_z;

float **Sample_X;
float **Sample_Y;
float **Sample_Z;

void initializePT()
{
	ptNum = 3966015;
	PT_x = new float[ptNum];
	PT_y = new float[ptNum];
	PT_z = new float[ptNum];
	for (int i = 0; i < ptNum; ++i) {
		PT_x[i] = ((float)rand()) / RAND_MAX * 2000;
		PT_y[i] = ((float)rand()) / RAND_MAX * 2000;
		PT_z[i] = ((float)rand()) / RAND_MAX * 2000;
	}
}

int main() {
	double start = omp_get_wtime();	
	srand ( time(NULL) );

	initializePT();		

	cout<<(omp_get_wtime() - start)<<" seconds."<<endl;

	double small = 200.0;
	double big = 250;
	double width = 10;
  	int sNum = 10000;

	intervalNum = (int)((big - small) / width);

	tempResult = new double*[numSamples];
	Sample_X = new float*[numSamples];
	Sample_Y = new float*[numSamples];
	Sample_Z = new float*[numSamples];

	double real_start = omp_get_wtime();	

	#pragma omp parallel for
	for (int s=0;s<numSamples;s++)
	{
		tempResult[s] = new double[intervalNum];
		for (int j=0;j<intervalNum;j++)
			tempResult[s][j]=0;

		// Get points
		Sample_X[s] = new float[sNum];
		Sample_Y[s] = new float[sNum];
		Sample_Z[s] = new float[sNum];

		for (int j = 0; j < sNum; ++j) {
			int index = (int)(((float)rand()) / RAND_MAX * ptNum);
			Sample_X[s][j] = PT_x[index];
			Sample_Y[s][j] = PT_y[index];
			Sample_Z[s][j] = PT_z[index];
		}

		double real_start = omp_get_wtime();	

		// Calculate
		for (int i=0;i<sNum-1;i++)
		for (int j=i+1;j<sNum;j++)
		{
			double dis = (Sample_X[s][i] - Sample_X[s][j]) * (Sample_X[s][i] - Sample_X[s][j]) +
			 (Sample_Y[s][i] - Sample_Y[s][j]) * (Sample_Y[s][i] - Sample_Y[s][j]) +
			 (Sample_Z[s][i] - Sample_Z[s][j]) * (Sample_Z[s][i] - Sample_Z[s][j]);

			dis = sqrt(dis);

			if (dis<small || dis > big)
				;
			else
			{
				int index = (int)((dis - small - 0.00000001) / width);
				tempResult[s][index] ++;
			}
		}

		//normalization
		for (int j=0;j<intervalNum;j++)
			tempResult[s][j] = tempResult[s][j] / sNum / sNum * ptNum * ptNum;
	}

	real_time += omp_get_wtime() - real_start;	

	// Statistic
	//calculate average
	for (int j = 0; j < intervalNum; j++) {
		double avg = 0.0;
		double std = 0.0;
	
		for (int i=0;i<numSamples;i++)
			avg += tempResult[i][j];
		avg /= numSamples;
					
		//calculate std
		for (int i=0;i<numSamples;i++)
			std += (tempResult[i][j]-avg)*(tempResult[i][j]-avg);
		std = std/numSamples/(numSamples-1);
		std = sqrt(std);			      

		//cout<<j<<","<<(small + j * width)<<","<<std<<","<<avg<<","<<std*2/avg<<endl;
	}

	cout<<(omp_get_wtime() - start)<<" seconds."<<endl;
	cout<<"Shuffle: "<<shuffle_time<<" seconds."<<endl;
	cout<<"Real: "<<real_time<<" seconds."<<endl;
}
